# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import json

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.contrib.auth.decorators import login_required, user_passes_test
from django.shortcuts import get_object_or_404

from balafon.Crm.models import Contact, Entity, Group, Action, Opportunity
from balafon.Search.models import Search
from balafon.permissions import can_access


def redirect_to_homepage(request):
    if getattr(settings, 'BALAFON_AS_HOMEPAGE', False):
        return HttpResponseRedirect(reverse('balafon_homepage'))
    else:
        return HttpResponseRedirect(reverse('coop_cms_homepage'))
        

@user_passes_test(can_access)
def auto_save_data(request, model_type, field_name, obj_id):
    if request.method != "POST" or (not request.is_ajax()):
        raise Http404
    try:
        model = {
            "group": Group,
            "contact": Contact,
            "entity": Entity,
            "action": Action,
            "opportunity": Opportunity,
            "search": Search,
        }[model_type]
        
        obj = get_object_or_404(model, id=obj_id)
        
        value = request.POST["value"]
        value = value.strip(" \t\r\n")
        if value[-4:] == "<br>":
            value = value[:-4]
        value = value.replace("<br>", "\n").replace("&nbsp;", " ")
        
        setattr(obj, field_name, value)
        obj.save()   
    except Http404:
        raise
    except Exception as msg:
        return HttpResponse(json.dumps({"ok": False, "error": str(msg)}), content_type='application/json')
    return HttpResponse(json.dumps({"ok": True}), content_type='application/json')



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-09 13:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Users', '0002_custommenuitem_reverse_kwargs'),
    ]

    operations = [
        migrations.AlterField(
            model_name='custommenu',
            name='icon',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='icon'),
        ),
        migrations.AlterField(
            model_name='custommenuitem',
            name='attributes',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='attributes'),
        ),
        migrations.AlterField(
            model_name='custommenuitem',
            name='icon',
            field=models.CharField(blank=True, default='', max_length=20, verbose_name='icon'),
        ),
        migrations.AlterField(
            model_name='custommenuitem',
            name='reverse',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='reverse'),
        ),
        migrations.AlterField(
            model_name='custommenuitem',
            name='reverse_kwargs',
            field=models.CharField(blank=True, default='', help_text='kwargs to use for building the reverse. kw1:defaultval1,kw2,kw3:defaultval3 ', max_length=100, verbose_name='reverse_kwargs'),
        ),
        migrations.AlterField(
            model_name='custommenuitem',
            name='url',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='url'),
        ),
    ]



# -*- coding: utf-8 -*-
"""REST api powered by django-rest-framework"""

from __future__ import unicode_literals

from decimal import Decimal
from datetime import date, datetime, time, timedelta

from django.db.models import Q
from django.shortcuts import get_object_or_404


from rest_framework.response import Response
from rest_framework.views import APIView

from balafon.Store.models import (
    SaleItem, StoreItem, StoreItemCategory, StoreItemTag, SaleAnalysisCode
)
from balafon.Store.api import get_staff_api_permissions, serializers


class SalesStatisticsBaseView(APIView):
    """get sales by category"""
    permission_classes = get_staff_api_permissions()

    def get_month_limits(self, the_date):
        """return the date limits of the given month"""
        date_from = datetime.combine(date(the_date.year, the_date.month, 1), time.min)

        next_month = date_from + timedelta(31)
        date_to = datetime.combine(date(next_month.year, next_month.month, 1), time.min)

        return date_from, date_to

    def get_sales_amount(self, sale_items_queryset):
        """returns the sum of sales for the given queryset"""
        total_amount = Decimal(0)
        for sales_item in sale_items_queryset:
            total_amount += sales_item.pre_tax_total_price()
        return total_amount

    def get_date_range(self, **kwargs):
        """date range from args"""
        from_month = int(kwargs['from_month'])
        from_year = int(kwargs['from_year'])
        to_month = int(kwargs['to_month'])
        to_year = int(kwargs['to_year'])

        first_date = datetime.combine(date(from_year, from_month, 1), time.min)
        last_date = datetime.combine(date(to_year, to_month, 1), time.min)
        return first_date, last_date

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        return []

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return SaleItem.objects.filter(
            sale__action__planned_date__gte=date_from,
            sale__action__planned_date__lt=date_to,
        ).exclude(sale__analysis_code__isnull=True)

    def get_object_name(self, obj):
        """return object name"""
        return obj.name

    def get_object_icon(self, obj):
        """return object name"""
        return obj.icon

    def get(self, request, **kwargs):
        """return a list with monthly amount by categories"""

        first_date, last_date = self.get_date_range(**kwargs)

        date_from, date_to = self.get_month_limits(first_date)

        objects = self.get_objects(**kwargs)

        objects_data = [
            {'id': obj.id, 'name': self.get_object_name(obj), 'icon': self.get_object_icon(obj), 'values': []}
            for obj in objects
        ]
        months = []
        while date_from <= last_date:
            months.append(date_from)

            for index, obj in enumerate(objects):
                sales_items = self.get_object_sale_items(obj, date_from, date_to)
                amount = self.get_sales_amount(sales_items)
                objects_data[index]['values'].append({'value': amount})

            next_date = date_to + timedelta(days=1)
            date_from, date_to = self.get_month_limits(next_date)

        stat_serializer = serializers.SalesStatisticSerializer(objects_data, many=True)
        months_serializer = serializers.DateSerializer(
            [{'date': _datetime} for _datetime in months], many=True
        )
        return Response({'months': months_serializer.data, 'data': stat_serializer.data})


class SalesByCategoryView(SalesStatisticsBaseView):
    """sales statistics by category"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        return list(StoreItemCategory.objects.filter(parent__isnull=False))

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return super(SalesByCategoryView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(item__category=obj)


class SalesByFamilyView(SalesStatisticsBaseView):
    """sales statistics by family"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        return list(StoreItemCategory.objects.filter(parent__isnull=True))

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return super(SalesByFamilyView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(Q(item__category__parent=obj) | Q(item__category=obj))


class SalesByTagView(SalesStatisticsBaseView):
    """sales statistics by tags"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        return list(StoreItemTag.objects.all())

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return super(SalesByTagView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(item__tags=obj)


class SalesByItemOfFamilyView(SalesStatisticsBaseView):
    """get sales by items of category"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        category = get_object_or_404(StoreItemCategory, id=kwargs['category_id'])
        return list(StoreItemCategory.objects.filter(parent=category))

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return super(SalesByItemOfFamilyView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(item__category=obj)


class SalesByItemOfCategoryView(SalesStatisticsBaseView):
    """get sales by items of category"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        category = get_object_or_404(StoreItemCategory, id=kwargs['category_id'])
        return list(StoreItem.objects.filter(category=category))

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return super(SalesByItemOfCategoryView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(item=obj)

    def get_object_icon(self, obj):
        """return object name"""
        return obj.category.icon if obj.category else ''


class SalesByItemOfTagView(SalesStatisticsBaseView):
    """get sales by items of tag"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        tag = get_object_or_404(StoreItemTag, id=kwargs['tag_id'])
        return list(StoreItem.objects.filter(tags=tag))

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""
        return super(SalesByItemOfTagView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(item=obj)

    def get_object_icon(self, obj):
        """return object name"""
        return obj.category.icon if obj.category else ''


class TotalSalesView(SalesStatisticsBaseView):
    """get sales by items of tag"""

    def get_objects(self, **kwargs):
        """return list of objects (a line of the statistic array)"""
        return SaleAnalysisCode.objects.all()

    def get_object_icon(self, obj):
        """return object name"""
        return 'piggy-bank'

    def get_object_sale_items(self, obj, date_from, date_to):
        """return sales for an objects for the given period (a cell of the statistic array)"""

        return super(TotalSalesView, self).get_object_sale_items(
            obj, date_from, date_to
        ).filter(sale__analysis_code=obj)





# -*- coding: utf-8 -*-
"""forms"""

from __future__ import unicode_literals

from datetime import date, datetime, time
from itertools import chain
import importlib
import json

from django.db.models import Q
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.forms.utils import flatatt
from django.template.loader import get_template
from django.utils.encoding import smart_text
from django.utils.html import escape
from django.utils.translation import ugettext as _

import floppyforms.__future__ as forms
from coop_cms.bs_forms import Form as BsForm

from balafon.fields import HidableModelMultipleChoiceField
from balafon.Crm.models import Contact, Action, Group, Subscription, SubscriptionType
from balafon.Crm.widgets import OpportunityAutoComplete
from balafon.Crm.utils import sort_by_entity_callback
from balafon.Search import models
from balafon.Search.widgets import DatespanInput
from balafon.Search.utils import get_date_bounds

SEARCH_FORMS = None


def load_from_name(constant_full_name):
    """load module dynamically"""
    constant_full_path = constant_full_name.split('.')
    constant_path, constant_name = '.'.join(constant_full_path[:-1]), constant_full_path[-1]
    module = importlib.import_module(constant_path)
    return getattr(module, constant_name)


class QuickSearchForm(BsForm):
    """Quick search form which is included in the menu"""
    text = forms.CharField(
        required=True,
        widget=forms.TextInput(attrs={'placeholder': _(u'Quick search')})
    )


def get_search_forms():
    """cache the configured forms"""
    global SEARCH_FORMS  #pylint: disable=global-statement
    if not SEARCH_FORMS:
        SEARCH_FORMS = load_from_name(settings.SEARCH_FORM_LIST)
    return SEARCH_FORMS


def get_field_form(field):
    """get field form"""
    _forms = []
    for cat_and_form in get_search_forms():
        _forms.extend(cat_and_form[1])
    field_dict = dict([(_form.name, _form) for _form in _forms if _form])
    return field_dict[field]


class GroupedSelect(forms.Select):
    """Build select with all forms"""
    def render(self, name, value, attrs=None, choices=(), *args, **kwargs):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs)
        final_attrs.update(dict(name=name))
        output = [u'<select {0}>'.format(flatatt(final_attrs))]
        str_value = smart_text(value)
        for group_label, group in self.choices: 
            if group_label:
                # should belong to an optgroup
                group_label = smart_text(group_label)
                output.append(u'<optgroup label="%s">' % escape(group_label)) 
            for key, value in group:
                # build option html
                option_value = smart_text(key)
                output.append(
                    u'<option value="{0}"{1}>{2}</option>'.format(
                        escape(option_value),
                        (option_value == str_value) and u' selected="selected"' or '',
                        escape(smart_text(value))
                    )
                )
            if group_label:
                output.append(u'</optgroup>')
        output.append(u'</select>') 
        return u'\n'.join(output)


class GroupedChoiceField(forms.ChoiceField):
    """GroupedChoiceField"""

    def __init__(self, choices=None, *args, **kwargs):

        kwargs.setdefault('required', True)
        kwargs.setdefault('widget', None)
        kwargs.setdefault('label', None)
        kwargs.setdefault('initial', None)
        kwargs.setdefault('help_text', None)

        super(GroupedChoiceField, self).__init__(*args, **kwargs)
        self.choices = choices or ()
        
    def clean(self, value):
        """
        Validates that the input is in self.choices.
        """
        value = super(GroupedChoiceField, self).clean(value)
        if value in (None, ''):
            value = u''
        value = smart_text(value)
        if value == u'':
            return value
        valid_values = []
        for choice in self.choices:
            group = choice[1]
            valid_values += [str(key) for key, value in group]
        if value not in valid_values:
            raise ValidationError(_(u'Select a valid choice. That choice is not one of the available choices.'))
        return value


class FieldChoiceForm(forms.Form):
    """The form for dynamically adding new filter"""
    
    def __init__(self, *args, **kwargs):
        super(FieldChoiceForm, self).__init__(*args, **kwargs)
        choices = [('', [('', '')])]
        for (cat, forms_) in get_search_forms():
            choices.append(
                (
                    cat,
                    [
                        (reverse('search_get_field', args=[form.name]), form.label)
                        for form in forms_ if form
                    ]
                )
            )
        widget = GroupedSelect(attrs={
            'class': 'form-control half-width',
            'data-placeholder': _(u'Please select a filter'),
        })
        self.fields['field_choice'] = GroupedChoiceField(choices, widget=widget)

    def as_it_is(self):
        """Returns this form rendered as HTML <p>s."""
        return self._html_output(
            normal_row=u'%(field)s<span>%(help_text)s</span>',
            error_row=u'%s',
            row_ender=u'',
            help_text_html=u' <span class="helptext">%s</span>',
            errors_on_separate_row=False
        )


class SearchForm(forms.Form):
    """Main search form"""
    name = forms.CharField(required=False, help_text=_('Enter a name and click save.'))
    excluded = forms.CharField(required=False, widget=forms.HiddenInput())

    class Media:
        """media files"""
        css = {
            'all': ('chosen/chosen.css', 'chosen/chosen-bootstrap.css')
        }
        js = (
            'chosen/chosen.jquery.js',
        )

    def __init__(self, data=None, instance=None, save=False, *args, **kwargs):
        super(SearchForm, self).__init__(data=data, *args, **kwargs)
        self._forms = {}
        self._instance = instance
        self._save = save
        self.contacts_display = False
        self.contains_refuse_newsletter = False
        if instance:
            self.fields['name'].initial = instance.name
        if not data and instance:
            data = {}
            for group in instance.searchgroup_set.all():
                for search_field in group.searchfield_set.all():
                    key = '-_-'.join((group.name, search_field.field, str(search_field.count or len(data))))
                    if search_field.is_list:
                        data[key] = self._str_to_list(search_field.value)
                    else:
                        data[key] = search_field.value

        if data:
            for key in data:
                if hasattr(data, 'getlist'):
                    value = data.getlist(key)
                    if len(value) == 1:
                        value = data.get(key)
                else:
                    value = data.get(key)
                try:
                    # extract search fields
                    group, field, fid = key.split('-_-')
                    # will raise an except for city visible field --> ignore this field
                    fid = int(fid)
                    if group not in self._forms:
                        self._forms[group] = []
                    form_class = get_field_form(field)
                    form = form_class(group, fid, {field: value})
                    self.contacts_display = self.contacts_display or form.contacts_display
                    self._forms[group].append(form)
                except ValueError:
                    pass
            # sort forms of a group according to their id
            for form in self._forms.values():
                form.sort(key=lambda field_: field_.count)
    
    def block_count(self):
        """number of block"""
        return len(self._forms)
    
    def field_count(self):
        """number of fields"""
        return sum([len(f) for f in self._forms])
    
    def _str_to_list(self, str_value):
        """convert string to list"""
        def _split_unicode(unicode_str):
            """remove trailing ' " from string"""
            unicode_str0 = unicode_str
            try:
                if unicode_str[:2] in ("u'", 'u"'):
                    unicode_str = unicode_str[2:]
                elif unicode_str[0] in ("'", '"'):
                    unicode_str = unicode_str[1:]
                
                if unicode_str[-1] in ("'", '"'):
                    unicode_str = unicode_str[:-1]
            except IndexError:
                return unicode_str0
            return unicode_str
            
        if str_value[0] == '[' and str_value[-1] == ']':
            # remove [ and ]
            values = str_value[1:-1]
            # get things separated by ', '
            values = values.split(", ")
            values = [value for value in values]
            values = [_split_unicode(value) for value in values]
            return values
        return []
    
    def serialize(self):
        """serialize the form for json"""
        data = {}
        for group, the_forms in self._forms.items():
            for form in the_forms:
                if group in data:
                    data[group] += [form.serialize()]
                else:
                    data[group] = [form.serialize()]
        return data
    
    def clean_name(self):
        """validate name"""
        name = self.cleaned_data['name']
        if len(name) >= 100:
            raise ValidationError(_("Too long"))
        if self._save:
            if name:
                queryset = models.Search.objects.filter(name=name)
                if self._instance and self._instance.id:
                    queryset = queryset.exclude(id=self._instance.id)
                if queryset.count() > 0:
                    raise ValidationError("This search name is already used")
            else:
                if self._instance:
                    raise ValidationError("A name is required for saving the search")
        return name
    
    def clean(self):
        """validate form"""
        keys = list(self._forms.keys())
        keys.sort()
        cleaned_data = super(SearchForm, self).clean()
        for key in keys:
            for form in self._forms[key]:
                cleaned_data.update(form.clean())
        return cleaned_data
    
    def save_search(self):
        """save search"""
        self._instance.searchgroup_set.all().delete()
        self._instance.name = self.cleaned_data['name']
        self._instance.save()
        keys = self._forms.keys()
        for key in keys:
            group = models.SearchGroup.objects.create(name=key, search=self._instance)
            for form in self._forms[key]:
                field = models.SearchField.objects.create(
                    search_group=group, field=form.name, value=form.value, count=form.count)
                if form.multi_values:
                    field.is_list = True
                    field.save()
        return self._instance
    
    def clean_excluded(self):
        """validate"""
        excluded = self.cleaned_data['excluded']
        if excluded:
            return [int(x) for x in excluded.strip("#").split("##")]
        return []

    def is_valid(self):
        """is full form valid?"""
        if not super(SearchForm, self).is_valid():
            return False
        keys = list(self._forms.keys())
        keys.sort()
        for key in keys:
            for form in self._forms[key]:
                if not form.is_valid():
                    return False
        return True
    
    def _get_contacts(self):
        """get contacts"""
        keys = list(self._forms.keys())
        keys.sort()
        contacts = set([])
        global_post_processors = []
        self.contains_refuse_newsletter = False
        has_sort_forms = False

        for key in keys:
            post_processors = []
            contacts_set = Contact.objects.all()
            actions_set = Action.objects.all()
            exclude_actions_set = Action.objects.all()
            has_action_forms = False
            has_exclude_action_forms = False

            if not ('secondary_contact' in [form.name for form in self._forms[key]]):
                contacts_set = contacts_set.filter(main_contact=True)
            
            for form in self._forms[key]:

                if form.is_sort_form:
                    has_sort_forms = True

                if not form.is_action_form:
                    contacts_set = form.get_queryset(contacts_set)
                else:
                    if form.is_exclude_action_form:
                        has_exclude_action_forms = True
                        exclude_actions_set = form.get_queryset(exclude_actions_set)
                    else:
                        has_action_forms = True
                        actions_set = form.get_queryset(actions_set)

                if hasattr(form, 'post_process'):
                    post_processors.append(form.post_process)
                    
                if hasattr(form, 'global_post_process'):
                    global_post_processors.append(form.global_post_process)

            if has_action_forms:
                contacts_set = contacts_set.filter(
                    Q(action__in=actions_set) | Q(entity__action__in=actions_set)
                ).distinct()

            if has_exclude_action_forms:
                contacts_set = contacts_set.exclude(
                    Q(action__in=exclude_actions_set) | Q(entity__action__in=exclude_actions_set)
                ).distinct()

            for post_processor in post_processors:
                contacts_set = post_processor(contacts_set)
                
            contacts = contacts.union(set(contacts_set))
        
        for global_post_processor in global_post_processors:
            contacts = global_post_processor(contacts)

        # By default sort by entity
        if not has_sort_forms:
            contacts = sorted(contacts, key=sort_by_entity_callback)

        # Just for compatibility
        current_site = Site.objects.get_current()
        queryset = SubscriptionType.objects.filter(Q(site=current_site) | Q(allowed_on_sites=current_site))
        if queryset.count():
            for contact in contacts:
                for subscription_type in queryset:
                    try:
                        if not contact.subscription_set.get(subscription_type=subscription_type).accept_subscription:
                            self.contains_refuse_newsletter = True
                            break
                    except Subscription.DoesNotExist:
                        self.contains_refuse_newsletter = True
                        break
                
        return list(contacts)
    
    def _get_filter_func(self):
        """filter function"""
        form_names = [form.name for form in chain.from_iterable(self._forms.values())]
        if 'contact_has_left' in form_names:
            return lambda contact: contact
        return lambda contact: contact and (not contact.has_left)

    def get_contacts_by_entity(self):
        """get contacts by entities"""
        contacts = self._get_contacts()
        contacts_count = 0
        entities = {}
        filter_func = self._get_filter_func()
        empty_entities = {}
        for contact in contacts:
            pass_filter = filter_func(contact)
            entity = contact.entity if contact else None
            if entity and not entity.id in entities:
                entities[entity.id] = (entity, [])
                empty_entities[entity.id] = entity
            if pass_filter:
                entities[entity.id][1].append(contact)
                entities[entity.id][1].sort(key=lambda c: c.lastname.lower())
                empty_entities.pop(entity.id, None)
                contacts_count += 1
            
        results = []
        for entity, contacts in entities.values():
            entity.search_contacts = contacts
            if entity.id in empty_entities:
                setattr(entity, 'is_empty', True)
            results.append(entity)
        results.sort(key=lambda x: x.name)
        return results, contacts_count, len(empty_entities) > 0
    
    def get_contacts(self):
        """get contacts"""
        filter_func = self._get_filter_func()
        contacts = self._get_contacts()
        contact_ids = self.cleaned_data['excluded']
        return [contact for contact in contacts if contact.id not in contact_ids and filter_func(contact)]
        
    def get_contacts_emails(self):
        """get contact emails"""
        filter_func = self._get_filter_func()
        contacts = self._get_contacts()
        excluded_ids = self.cleaned_data['excluded']
        emails = []
        for contact in contacts:
            if contact.get_email and (contact.id not in excluded_ids) and filter_func(contact):
                if contact.firstname or contact.lastname:
                    emails.append('"{1}" <{0}>'.format(contact.get_email, contact.fullname))
                else:
                    emails.append(contact.get_email)
        return emails
    
    def actions(self):
        """allowed actions"""
        form = FieldChoiceForm()
        html_tpl = """{0}
            <a class="btn btn-xs btn-default btn-yellow add-field" href="">
            <span class="glyphicon glyphicon-filter"></span> {1}</a>
            <a class="btn btn-xs btn-default add-block" href="">
            <span class="glyphicon glyphicon-th-list"></span> {2}</a>
            <a class="btn btn-xs btn-default duplicate-block" href="">
            <span class="glyphicon glyphicon-share"></span> {5}</a>
            <a class="btn btn-xs btn-danger clear-block" href="">
            <span class="glyphicon glyphicon-remove"></span> {3}</a>
            <a class="btn btn-xs btn-danger remove-block" href="">
            <span class="glyphicon glyphicon-trash"></span> {4}</a>"""

        return html_tpl.format(
            form.as_it_is(), _('Add filter'), _('Add block'), _('Clear'), _('Remove'), _('Duplicate')
        )
    
    def as_html(self):
        """as html"""
        keys = list(self._forms.keys())
        keys.sort()
        html = '<div>{0}</div>'.format(self.as_p())
        if keys:
            for key in keys:
                html_tpl = '<div class="search-block" rel="{0}"><div class="actions">{1}</div><div class="fields">'
                html += html_tpl.format(key, self.actions())
                for form in self._forms[key]:
                    html += form.as_it_is()
                html += '</div></div>'
        return html


class SearchFieldForm(BsForm):
    """Base class for search forms"""
    contacts_display = False
    multi_values = False
    is_action_form = False
    is_exclude_action_form = False
    is_sort_form = False

    def __init__(self, block, count, data=None, *args, **kwargs):
        self.block = block
        self.count = count
        form_data = None
        if data:
            if self.multi_values:
                val = data[self.name]
                self.value = val if type(val) is list else [val]
            else:
                self.value = data[self.name]
            form_data = {self._get_field_name(): self.value}
        super(SearchFieldForm, self).__init__(form_data, *args, **kwargs)
        
    def _get_field_name(self):
        """return field name"""
        return self.block + '-_-' + self.name + '-_-' + str(self.count)
        
    def _add_field(self, field):
        """adda field"""
        field.required = True
        if not field.widget.attrs.get('class', None):
            field.widget.attrs['class'] = "form-control"
        else:
            field.widget.attrs['class'] += " form-control"
        self.fields[self._get_field_name()] = field
        
    def clean(self):
        """return cleaned data"""
        return {self._get_field_name(): self.value}
        
    def serialize(self):
        """serialize"""
        return {self.name: self.value}
        
    def as_it_is(self):
        """return form html without any wrapper tag"""
        template = get_template("Search/_search_field_form.html")
        return template.render({"form": self})
    
    def get_queryset(self, queryset):
        """
        returns a queryset for filtering contacts corresponding to searches
        it can be overriden in subclassses
        A subclass may also define a get_lookup or get_exclude_lookup method for building the queryset more easily
        (and for compatibility reasons)
        """
        if hasattr(self, 'get_lookup'):
            lookup = self.get_lookup()
            if lookup:
                if type(lookup) is dict:
                    queryset = queryset.filter(**lookup)
                elif type(lookup) is list:
                    queryset = queryset.filter(*lookup)
                else:
                    queryset = queryset.filter(lookup)
                    
        if hasattr(self, 'get_exclude_lookup'):
            lookup = self.get_exclude_lookup()
            if lookup:
                if type(lookup) is dict:
                    queryset = queryset.exclude(**lookup)
                elif type(lookup) is list:
                    queryset = queryset.exclude(*lookup)
                else:
                    queryset = queryset.exclude(lookup)

        return queryset


class TwoDatesForm(SearchFieldForm):
    """Base class for any SerachForm providing a 'TwoDate' field"""
    
    def __init__(self, *args, **kwargs):
        super(TwoDatesForm, self).__init__(*args, **kwargs)
        field = forms.CharField(
            label=self.label, initial='{0} {0}'.format(date.today().strftime("%d/%m/%Y")),
            widget=DatespanInput()
        )
        self._add_field(field)
        
    def __getattr__(self, name):
        """get attribute dynamically"""
        if name == 'clean_'+self._get_field_name():
            return self._clean_field
        return super(TwoDatesForm, self).__getattr__(name)
    
    def _clean_field(self):
        """validate"""
        try:
            self._get_dates()
        except ValueError:
            raise ValidationError(_("Two valid dates are required"))
        return self.value
    
    def _get_dates(self):
        """return selected dates"""
        return get_date_bounds(self.value)

    def _get_datetimes(self):
        """return selected date times"""
        start_date, end_date = get_date_bounds(self.value)
        return datetime.combine(start_date, time.min), datetime.combine(end_date, time.max)


class YesNoSearchFieldForm(SearchFieldForm):
    """Base class for ny SearchForm providing a Yes/No field"""

    def __init__(self, *args, **kwargs):
        super(YesNoSearchFieldForm, self).__init__(*args, **kwargs)
        choices = ((1, _('Yes')), (0, _('No')),)
        field = forms.ChoiceField(choices=choices, label=self.label)
        self._add_field(field)
    
    def is_yes(self):
        """Is yes selected?"""
        return True if int(self.value) else False


class SearchActionBaseMixin(object):
    """
    Base class for any views which needs to get extra information for handling results
    Example : Pdf, CreateActions...
    """
    def _pre_init(self, *args, **kwargs):
        """at the beginning of __init__"""
        initial = kwargs.get('initial')
        initial_contacts = ''
        if initial and 'contacts' in initial:
            initial_contacts = ';'.join(['{0}'.format(contact.id) for contact in initial['contacts']])
            initial.pop('contacts')
        return initial_contacts
        
    def _post_init(self, initial_contacts):
        """at the end of __init__"""
        if initial_contacts:
            self.fields['contacts'].initial = initial_contacts

    def get_contacts(self):
        """get contacts"""
        ids = self.cleaned_data["contacts"].split(";")
        contacts = Contact.objects.filter(id__in=ids)
        contact_by_ids = {}
        for contact in contacts:
            contact_by_ids['{0}'.format(contact.id)] = contact
        ordered_contacts = []
        for contact_id in ids:
            ordered_contacts.append(contact_by_ids[contact_id])
        return ordered_contacts


class SearchActionForm(BsForm, SearchActionBaseMixin):
    """Base class for any views which needs to get extra information for handling results"""
    contacts = forms.CharField(widget=forms.HiddenInput())
    
    def __init__(self, *args, **kwargs):
        initial_contacts = self._pre_init(*args, **kwargs)
        super(SearchActionForm, self).__init__(*args, **kwargs)
        self._post_init(initial_contacts)


class SubscribeContactsAdminForm(SearchActionForm):
    """This form is used for superuser forcing newsletter subscription"""
    subscription_type = forms.ChoiceField(required=True)
    subscribe = forms.BooleanField(
        required=False, label=_('Subscribe'), help_text=_('It will subscribe/unsubscribe all selected contacts')
    )

    def __init__(self, *args, **kwargs):
        super(SubscribeContactsAdminForm, self).__init__(*args, **kwargs)
        self.fields['subscription_type'].choices = [
            (subscription_type.id, subscription_type.name)
            for subscription_type in SubscriptionType.objects.all()
        ]

    def clean_subscription_type(self):
        try:
            return SubscriptionType.objects.get(id=self.cleaned_data['subscription_type'])
        except SubscriptionType.DoesNotExist:
            raise ValidationError(_('Unknown subscription type'))


class PdfTemplateForm(SearchActionForm):
    """Form for Pdf generation"""
    search_dict = forms.CharField(widget=forms.HiddenInput())
    
    def __init__(self, *args, **kwargs):
        super(PdfTemplateForm, self).__init__(*args, **kwargs)
        
        if getattr(settings, 'BALAFON_PDF_TEMPLATES', None):
            choices = settings.BALAFON_PDF_TEMPLATES
        else:
            choices = (
                ('pdf/labels_24.html', _('etiquettes 24')),
                ('pdf/labels_21.html', _('etiquettes 21')),
                ('pdf/agipa_21.html', _('Agipa 21')),
                ('pdf/labels_16.html', _('etiquettes 16')),
                ('pdf/address_strip.html', _('bande adresse')),
            )
        
        self.fields['template'] = forms.ChoiceField(
            choices=choices,
            required=True,
            label=_('template'),
            help_text=_('Select the type of document to generate')
        )
        self.fields['template'].widget.attrs.update({'class': 'form-control'})
        
        extra_fields = getattr(settings, 'BALAFON_PDF_FORM_EXTRA_FIELDS', None)
        if extra_fields:
            for field_name, field_label, initial_value in extra_fields:
                self.fields[field_name] = forms.CharField(label=field_label, initial=initial_value)
                self.fields[field_name].widget.attrs.update({'class': 'form-control'})
       
    def patch_context(self, context):
        """
        add to data from external function in context
        It can be used for example for adding blank labels in labels generation
        """
        template = self.cleaned_data['template']
        extra_data = self._get_extra_data()
        context.update(extra_data)
        hooks = getattr(settings, 'BALAFON_PDF_FORM_CONTEXT_HOOKS', {})
        if hooks and template in hooks:
            context = hooks[template](template, context)
        return context
    
    def _get_extra_data(self):
        """
        This extra_data comes from additional fields defined in BALAFON_PDF_FORM_EXTRA_FIELDS settings
        These values are passed to the template
        """
        extra_data = dict(self.cleaned_data)
        for field in ('template', 'contacts', 'search_dict'):
            extra_data.pop(field)
        return extra_data
        
        
class ActionForContactsForm(forms.ModelForm):
    """Create action for contacts"""
    date = forms.DateField(label=_("planned date"), required=False, widget=forms.TextInput())
    time = forms.TimeField(label=_("planned time"), required=False)
    contacts = forms.CharField(widget=forms.HiddenInput())
    
    class Meta:
        """create form from model"""
        model = Action
        fields = (
            'date', 'time', 'type', 'subject', 'in_charge', 'detail', 'planned_date', 'contacts', 'opportunity'
        )
        
    def __init__(self, *args, **kwargs):
        initial = kwargs.get('initial')
        initial_contacts = ''
        if initial and 'contacts' in initial:
            initial_contacts = ';'.join(['{0}'.format(c.id) for c in initial['contacts']])
            initial.pop('contacts')
        super(ActionForContactsForm, self).__init__(*args, **kwargs)
        if initial_contacts:
            self.fields['contacts'].initial = initial_contacts
        self.fields['opportunity'].widget = OpportunityAutoComplete(
            attrs={'placeholder': _('Enter the name of an opportunity'), 'size': '80', 'class': 'colorbox'}
        )
        
    def get_contacts(self):
        """return contacts"""
        contact_ids = self.cleaned_data["contacts"].split(";")
        return Contact.objects.filter(id__in=contact_ids)
        
    def clean_planned_date(self):
        """validate planned date"""
        the_date = self.cleaned_data["date"]
        the_time = self.cleaned_data.get("time", None)
        if the_date:
            return datetime.combine(the_date, the_time or datetime.min.time())
        return None


class GroupForContactsForm(forms.Form):
    """Add contacts to group"""
    contacts = forms.CharField(widget=forms.HiddenInput())
    groups = HidableModelMultipleChoiceField(queryset=Group.objects.all())
    on_contact = forms.BooleanField(
        label=_("Group on contact"),
        required=False,
        initial=True,
        help_text=_("Define if the group is added on the contact itself or on his entity")
    )
    
    class Media:
        """media files"""
        css = {
            'all': ('chosen/chosen.css',)
        }
        js = (
            'chosen/chosen.jquery.js',
        )
        
    def __init__(self, *args, **kwargs):
        initial = kwargs.get('initial')
        initial_contacts = ''
        if initial and 'contacts' in initial:
            initial_contacts = ';'.join(['{0}'.format(contact.id) for contact in initial['contacts']])
            initial.pop('contacts')
        super(GroupForContactsForm, self).__init__(*args, **kwargs)
        if initial_contacts:
            self.fields['contacts'].initial = initial_contacts
    
        self.fields['groups'].widget.attrs = {
            'class': 'chzn-select',
            'data-placeholder': _('Select groups'),
            'style': "width:350px;"
        }
        self.fields['groups'].help_text = ''

    def get_contacts(self):
        """get contacts"""
        contact_ids = self.cleaned_data["contacts"].split(";")
        return Contact.objects.filter(id__in=contact_ids)


class SearchNameForm(forms.Form):
    """Save search form"""
    search_id = forms.IntegerField(required=False, widget=forms.HiddenInput())
    name = forms.CharField(required=True, label=_("Name"))
    search_fields = forms.CharField(required=True, widget=forms.HiddenInput())
    
    def clean_name(self):
        """validate name"""
        search_id = self.cleaned_data["search_id"]
        
        name = self.cleaned_data["name"]
        if not name:
            raise ValidationError(_("This field is required"))
        
        queryset = models.Search.objects.filter(name=name)
        if search_id:
            queryset = queryset.exclude(id=search_id)
        if queryset.count() > 0:
            raise ValidationError(_("This name is already used"))
        
        if search_id:
            search = models.Search.objects.get(id=search_id)
            if search.name != name:
                search.name = name
                search.save()
        else:
            search = models.Search(name=name)        
        return search
    
    def clean_search_fields(self):
        """validate search fields"""
        search_fields = self.cleaned_data["search_fields"]
        data = json.loads(search_fields)
        for key in ('csrfmiddlewaretoken', 'excluded', 'name', 'field_choice'):
            data.pop(key, None)
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0019_auto_20180115_1751'),
    ]

    operations = [
        migrations.AddField(
            model_name='actiontype',
            name='is_default',
            field=models.BooleanField(default=False, help_text='If checked, can be added from list. Action without types are not displayed', verbose_name='is default'),
        ),
        migrations.AlterField(
            model_name='actiontype',
            name='track_status',
            field=models.BooleanField(default=False, help_text='If checked, all status changed is stored in DB', verbose_name='Track status'),
        ),
        migrations.AlterField(
            model_name='opportunity',
            name='display_on_board',
            field=models.BooleanField(default=False, db_index=True, verbose_name='display on board'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import floppyforms.__future__ as forms


class CityAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/city_autocomplete.html'


class CityNoCountryAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/city_nocountry_autocomplete.html'


class OpportunityAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/opportunity_autocomplete.html'


class EntityAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/entity_autocomplete.html'


class ContactAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/contact_autocomplete.html'


class ContactOrEntityAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/contact_or_entity_autocomplete.html'


class GroupAutoComplete(forms.HiddenInput):
    input_type = 'text'
    is_hidden = False
    template_name = 'forms/group_autocomplete.html'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0009_storemanagementactiontype_references_text'),
    ]

    operations = [
        migrations.AddField(
            model_name='storeitem',
            name='published',
            field=models.BooleanField(default=True, verbose_name='Published'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Crm forms"""

from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext, ugettext_lazy as _

import floppyforms.__future__ as forms

from coop_cms.bs_forms import ModelForm as BsModelForm

from balafon.Crm import models
from balafon.Crm.forms.base import ModelFormWithAddress, FormWithFieldsetMixin, BetterBsForm
from balafon.Crm.settings import (
    get_language_choices, has_language_choices, get_subscription_default_value, show_billing_address,
    ALLOW_COUPLE_GENDER,
)
from balafon.Crm.utils import get_suggested_same_as_contacts
from balafon.Crm.widgets import ContactAutoComplete


class ContactForm(FormWithFieldsetMixin, ModelFormWithAddress):
    """Edit contact form"""
    same_as_suggestions = forms.ChoiceField(
        label=_('Same-as contacts'),
        choices=[],
        required=False,
        help_text=_(
            'If this is a new contact for an existing person, you can link the different contacts between them'
        ),
        widget=forms.Select(
            attrs={'class': 'chosen-select', 'data-placeholder': _('Select same-as contacts'), 'style': "width: 100%;"}
        )
    )

    class Meta:
        """form is defined from model"""
        model = models.Contact
        fields = (
            'gender', 'gender_title', 'lastname', 'firstname', 'birth_date', 'title', 'role', 'job',
            'email', 'phone', 'mobile', 'favorite_language',
            'street_number', 'street_type', 'address', 'address2', 'address3', 'zip_code', 'city', 'cedex', 'country',
            'main_contact', 'email_verified', 'has_left', 'accept_notifications', 'photo',
            'billing_street_number', 'billing_street_type', 'billing_address', 'billing_address2', 'billing_address3',
            'billing_zip_code', 'billing_city', 'billing_cedex', 'billing_country', 'same_as_suggestions'
        )
        widgets = {
            'notes': forms.Textarea(attrs={'placeholder': _('enter notes about the contact'), 'cols': '72'}),
            'role': forms.SelectMultiple(
                attrs={'class': 'chosen-select', 'data-placeholder': _('Select roles'), 'style': "width: 100%;"}
            ),
        }
        fieldsets = [
            ('name', {
                'fields': [
                    'gender', 'gender_title', 'lastname', 'firstname', 'email', 'same_as_suggestions', 'phone', 'mobile'
                ],
                'legend': _('Name')
            }),
            ('web', {
                'fields': [
                    'birth_date', 'title', 'role', 'job', 'favorite_language',
                ],
                'legend': _('Contact details')
                }
            ),
            ('address', {
                'fields': [
                    'street_number', 'street_type', 'address', 'address2', 'address3', 'zip_code', 'city',
                    'cedex', 'country'
                ],
                'legend': _('Address')
            }),
            ('billing_address', {
                'fields': [
                    'billing_street_number', 'billing_street_type', 'billing_address', 'billing_address2',
                    'billing_address3', 'billing_zip_code', 'billing_city', 'billing_cedex', 'billing_country'
                ],
                'legend': _('Billing address')
            }),
            ('relationship', {
                'fields': ['main_contact', 'email_verified', 'has_left', 'accept_notifications'],
                'legend': _('Options')
            }),
            ('photo', {'fields': ['photo'], 'legend': _('Photo')}),
        ]

    def __init__(self, *args, **kwargs):
        # Configure the fieldset with dynamic fields
        fieldset_fields = self.Meta.fieldsets[-2][1]["fields"]
        for subscription_type in models.SubscriptionType.objects.all():
            field_name = "subscription_{0}".format(subscription_type.id)
            if field_name not in fieldset_fields:
                fieldset_fields.append(field_name)

        has_data = len(args) > 0
        super(ContactForm, self).__init__(*args, **kwargs)

        try:
            if self.instance and self.instance.entity and self.instance.entity.is_single_contact:
                self.fields['has_left'].widget = forms.HiddenInput()
        except models.Entity.DoesNotExist:
            pass

        self.fields["role"].help_text = _("Select the roles played by the contact in his entity")

        if 'balafon.Profile' not in settings.INSTALLED_APPS:
            self.fields["accept_notifications"].widget = forms.HiddenInput()

        if has_data:
            self.fields.pop("email_verified")
        else:
            self.fields["email_verified"].widget.attrs['disabled'] = "disabled"

        # define the allowed gender
        gender_choices = [
            (models.Contact.GENDER_NOT_SET, '-------'),
            (models.Contact.GENDER_MALE, ugettext('Mr')),
            (models.Contact.GENDER_FEMALE, ugettext('Mrs')),
        ]
        if ALLOW_COUPLE_GENDER:
            gender_choices += [
                (models.Contact.GENDER_COUPLE, ugettext('Mrs and Mr'))
            ]
        self.fields['gender'].choices = gender_choices

        # create the dynamic fields
        for subscription_type in models.SubscriptionType.objects.all():
            field_name = "subscription_{0}".format(subscription_type.id)
            field = self.fields[field_name] = forms.BooleanField(
                label=subscription_type.name, required=False
            )
            if self.instance and self.instance.id:
                try:
                    subscription = models.Subscription.objects.get(
                        subscription_type=subscription_type,
                        contact=self.instance
                    )
                    field.initial = subscription.accept_subscription
                except models.Subscription.DoesNotExist:
                    field.initial = False
            else:
                field.initial = get_subscription_default_value()

        if has_language_choices():
            self.fields['favorite_language'].widget = forms.Select(
                choices=get_language_choices(), attrs={'class': 'form-control'}
            )
        else:
            self.fields['favorite_language'].widget = forms.HiddenInput()

        if not self.instance or not any([self.instance.lastname, self.instance.firstname, self.instance.email]):
            # If the contact has not been created of not filled
            pass
            # keep the widget
            if len(args):
                # The form has been posted; we must initialize the list with allowed values
                contact_id = self.instance.id if self.instance else None
                post_data = args[0]
                lastname = post_data.get('lastname', '')
                firstname = post_data.get('firstname', '')
                email = post_data.get('email', '')
                same_as_contacts = get_suggested_same_as_contacts(
                    contact_id=contact_id, lastname=lastname, firstname=firstname, email=email
                )
                self.fields['same_as_suggestions'].choices = [
                    (contact.id, '{0}'.format(contact)) for contact in same_as_contacts
                ]
        else:
            # hide the field
            self.fields['same_as_suggestions'].widget = forms.HiddenInput()

    def get_fieldsets(self):
        """return the list of fieldsets"""
        for fieldset in self.Meta.fieldsets:
            if fieldset[0] == 'billing_address' and not show_billing_address():
                # Ignore 'Billing address' tab if disable in settings
                continue
            yield fieldset

    def clean_photo(self):
        """photo validation"""
        photo = self.cleaned_data["photo"]
        instance = self.instance
        if not instance:
            instance = ""
            try:
                instance.id = models.Contact.objects.latest('id').id
            except models.Contact.DoesNotExist:
                instance.id = 1
        target_name = models.get_contact_photo_dir(instance, photo)
        if len(target_name) >= models.Contact._meta.get_field('photo').max_length:
            raise ValidationError(ugettext("The file name is too long"))
        return photo

    def save_contact_subscriptions(self, contact):
        """save contact subscriptions"""
        for subscription_type in models.SubscriptionType.objects.all():
            field_name = "subscription_{0}".format(subscription_type.id)
            accept_subscription = self.cleaned_data[field_name]
            try:
                subscription = models.Subscription.objects.get(
                    contact=contact, subscription_type=subscription_type
                )
                if subscription.accept_subscription != accept_subscription:
                    subscription.accept_subscription = accept_subscription
                    subscription.save()
            except models.Subscription.DoesNotExist:
                if accept_subscription:
                    models.Subscription.objects.create(
                        contact=contact,
                        subscription_type=subscription_type,
                        accept_subscription=True
                    )

    def clean_same_as_suggestions(self):
        contact_id = self.cleaned_data.get('same_as_suggestions', '')
        if contact_id:
            try:
                contact = models.Contact.objects.get(id=contact_id)
                return contact
            except models.Contact.DoesNotExist:
                raise ValidationError(ugettext("Contact does'nt exist"))

    def save_same_as(self, this_contact):
        contact = self.cleaned_data['same_as_suggestions']
        if contact:
            # if not found create one
            if not contact.same_as:
                this_contact.same_as = models.SameAs.objects.create()
                this_contact.same_as_priority = 2
                contact.same_as = this_contact.same_as
                contact.same_as_priority = 1
                contact.save()
            else:
                this_contact.same_as = contact.same_as
                this_contact.same_as_priority = contact.same_as.contact_set.count() + 1
            this_contact.save()

    def save(self, *args, **kwargs):
        """save"""
        contact = super(ContactForm, self).save(*args, **kwargs)
        if kwargs.get('commit', True):
            self.save_contact_subscriptions(contact)
            self.save_same_as(contact)
        return contact


class SelectContactForm(forms.Form):
    """Select a contact"""

    def __init__(self, *args, **kwargs):
        choices = kwargs.pop('choices', None)
        super(SelectContactForm, self).__init__(*args, **kwargs)
        if choices:
            widget = forms.Select(choices=[(x.id, x.fullname) for x in choices])
            self.fields["contact"] = forms.IntegerField(label=_("Contact"), widget=widget)
        else:
            widget = ContactAutoComplete(
                attrs={'placeholder': _('Enter the name of a contact'), 'size': '50', 'class': 'colorbox'}
            )
            self.fields["contact"] = forms.CharField(label=_("Contact"), widget=widget)

    def clean_contact(self):
        """validation"""
        try:
            contact_id = int(self.cleaned_data["contact"])
            return models.Contact.objects.get(id=contact_id)
        except (ValueError, models.Contact.DoesNotExist):
            raise ValidationError(ugettext("The contact does'nt exist"))


class SelectContactOrEntityForm(forms.Form):
    """Select a contact or entity"""
    object_id = forms.IntegerField(widget=forms.HiddenInput(), required=True)
    object_type = forms.CharField(widget=forms.HiddenInput(), required=True)
    name = forms.CharField(required=True, label=_('Name'))

    def clean_name(self):
        """validation"""
        object_id = self.cleaned_data.get('object_id', '')
        object_type = self.cleaned_data.get('object_type', '')
        object_class = {
            'contact': models.Contact,
            'entity': models.Entity,
        }.get(object_type, None)

        if not object_class:
            raise ValidationError(ugettext("Invalid type '{0}'").format(object_type))

        try:
            object_id = int(object_id)
            return object_class.objects.get(id=object_id)
        except (ValueError, object_class.DoesNotExist):
            raise ValidationError(ugettext("Does'nt exist"))


class SameAsPriorityForm(BetterBsForm):
    """Define "same as" priority for a contact"""
    priority = forms.IntegerField(label=_("Priority"))

    def __init__(self, contact, *args, **kwargs):
        super(SameAsPriorityForm, self).__init__(*args, **kwargs)
        self.contact = contact
        self.fields['priority'].initial = contact.same_as_priority

    def clean_priority(self):
        """make sure that the priority is an integer between 1 and number of same as"""
        value = self.cleaned_data['priority']
        min_value = 1
        max_value = self.contact.same_as.contact_set.count()
        if value < min_value or value > max_value:
            raise ValidationError(
                ugettext('Invalid value : It should be between {0} and {1}').format(
                    min_value, max_value
                )
            )
        return value


class SameAsSuggestionForm(forms.Form):
    """
    generate suggestion list from a few data.
    This form is not displayed and used by json API
    """
    id = forms.IntegerField(required=False)
    lastname = forms.CharField(required=False)
    firstname = forms.CharField(required=False)
    email = forms.CharField(required=False)

    def get_suggested_contacts(self):
        """return list a suggestions for same as"""
        contact_id = self.cleaned_data['id']
        lastname = self.cleaned_data['lastname']
        firstname = self.cleaned_data['firstname']
        email = self.cleaned_data['email']
        return get_suggested_same_as_contacts(
            contact_id=contact_id, lastname=lastname, firstname=firstname, email=email
        )


class SameAsForm(forms.Form):
    """Define "same as" for two contacts"""
    contact = forms.IntegerField(label=_("Contact"))

    def __init__(self, contact, *args, **kwargs):
        super(SameAsForm, self).__init__(*args, **kwargs)
        potential_contacts = get_suggested_same_as_contacts(
            contact_id=contact.id, lastname=contact.lastname, firstname=contact.firstname, email=contact.email
        )
        if contact.same_as:
            # Do not propose again current SameAs
            potential_contacts = potential_contacts.exclude(same_as=contact.same_as)
        self._same_as = [(same_as_contact.id, "{0}".format(same_as_contact)) for same_as_contact in potential_contacts]
        if len(self._same_as):
            self.fields["contact"].widget = forms.Select(choices=self._same_as)
        else:
            self.fields["contact"].widget = forms.HiddenInput()

    def has_choices(self):
        """true if several contacts with same name"""
        return len(self._same_as)

    def clean_contact(self):
        """validation"""
        contact_id = self.cleaned_data["contact"]
        try:
            if contact_id not in [same_as[0] for same_as in self._same_as]:
                raise ValidationError(ugettext("Invalid contact"))
            return models.Contact.objects.get(id=contact_id)
        except models.Contact.DoesNotExist:
            raise ValidationError(ugettext("Contact does not exist"))


class AddRelationshipForm(forms.Form):
    """form for adding relationships"""
    relationship_type = forms.IntegerField(label=_("relationship type"))
    contact2 = forms.CharField(label=_("Contact"))

    def __init__(self, contact1, *args, **kwargs):
        super(AddRelationshipForm, self).__init__(*args, **kwargs)

        self.reversed_relation = False
        self.contact1 = contact1

        relationship_types = []
        for relationship_type in models.RelationshipType.objects.all():
            relationship_types.append((relationship_type.id, relationship_type.name))
            if relationship_type.reverse:
                relationship_types.append((-relationship_type.id, relationship_type.reverse))
        self.fields["relationship_type"].widget = forms.Select(choices=relationship_types)

        widget = ContactAutoComplete(
            attrs={'placeholder': _('Enter the name of a contact'), 'size': '50', 'class': 'colorbox'}
        )
        self.fields["contact2"] = forms.CharField(label=_("Contact"), widget=widget)

    def clean_relationship_type(self):
        """validate relationship type"""
        try:
            self.reversed_relation = False
            relationship_type = int(self.cleaned_data["relationship_type"])
            if relationship_type < 0:
                self.reversed_relation = True
                relationship_type = -relationship_type
            return models.RelationshipType.objects.get(id=relationship_type)
        except ValueError:
            raise ValidationError(ugettext("Invalid data"))
        except models.RelationshipType.DoesNotExist:
            raise ValidationError(ugettext("Relationship type does not exist"))

    def clean_contact2(self):
        """clean the contacts in relation"""
        try:
            contact2 = int(self.cleaned_data["contact2"])
            return models.Contact.objects.get(id=contact2)
        except ValueError:
            raise ValidationError(ugettext("Invalid data"))
        except models.Contact.DoesNotExist:
            raise ValidationError(ugettext("Contact does not exist"))

    def save(self):
        """save"""
        if self.reversed_relation:
            contact1 = self.cleaned_data["contact2"]
            contact2 = self.contact1
        else:
            contact1 = self.contact1
            contact2 = self.cleaned_data["contact2"]

        relationship_type = self.cleaned_data["relationship_type"]
        return models.Relationship.objects.create(
            contact1=contact1, contact2=contact2, relationship_type=relationship_type
        )


class ContactsImportForm(BsModelForm):
    """form for importing data"""
    class Meta:
        """form from model"""
        model = models.ContactsImport
        fields = ('import_file', 'name', 'encoding', 'separator', 'entity_type', 'groups', 'entity_name_from_email', )

    class Media:
        """media files"""
        css = {
            'all': ('chosen/chosen.css', 'chosen/chosen-bootstrap.css')
        }
        js = (
            'chosen/chosen.jquery.js',
        )

    def __init__(self, *args, **kwargs):
        super(ContactsImportForm, self).__init__(*args, **kwargs)

        self.fields['groups'].widget.attrs = {
            'class': 'chosen-select form-control',
            'data-placeholder': _('The created entities will be added to the selected groups'),
        }
        self.fields['groups'].help_text = ''

    def clean_separator(self):
        """validation"""
        if len(self.cleaned_data["separator"]) != 1:
            raise ValidationError(ugettext('Invalid separator {0}').format(self.cleaned_data["separator"]))
        return self.cleaned_data["separator"]


class ContactsImportConfirmForm(ContactsImportForm):
    """confirm contact import"""
    default_department = forms.ChoiceField(
        required=False,
        label=_('Default department'),
        help_text=_('The city in red will be created with this department as parent')
    )

    class Meta(ContactsImportForm.Meta):
        """from model"""
        fields = ('encoding', 'separator', 'entity_type', 'groups', 'entity_name_from_email', )

    def __init__(self, *args, **kwargs):
        super(ContactsImportConfirmForm, self).__init__(*args, **kwargs)
        zone_tuples = [(zone.code, zone.name) for zone in models.Zone.objects.filter(type__type='department')]
        self.fields['default_department'].choices = [('', '')] + zone_tuples

    def clean_default_department(self):
        """validation"""
        code = self.cleaned_data['default_department']
        if code:
            if not models.Zone.objects.filter(code=self.cleaned_data['default_department']):
                raise ValidationError(ugettext('Please enter a valid code'))
            return self.cleaned_data['default_department']
        else:
            return None


class UnsubscribeContactsImportForm(forms.Form):
    """A form for uploading a file"""
    input_file = forms.FileField()



# -*- coding: utf-8 -*-
"""test relationship search"""

from __future__ import unicode_literals

from datetime import date, timedelta

from django.core.urlresolvers import reverse

from model_mommy import mommy

from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class RelationshipSearchTest(BaseTestCase):
    """search on relationship"""

    def test_relationship_with_reverse(self):
        """contacts by relationship type"""

        anakin = mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker")
        obi = mommy.make(models.Contact, firstname="Obi-Wan", lastname="Kenobi")
        doe = mommy.make(models.Contact, firstname="Jack", lastname="Doedoedoe")

        master = mommy.make(models.RelationshipType, name="Master", reverse="Padawan")

        models.Relationship.objects.create(contact1=obi, contact2=anakin, relationship_type=master)

        url = reverse('search')

        data = {"gr0-_-contacts_by_relationship_type-_-0": master.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, obi.lastname)
        self.assertNotContains(response, anakin.lastname)
        self.assertNotContains(response, doe.lastname)

    def test_relationship_reverse(self):
        """contacts by relationship type: reverse"""

        anakin = mommy.make(models.Contact, firstname="Anakin", lastname="Skywalker")
        obi = mommy.make(models.Contact, firstname="Obi-Wan", lastname="Kenobi")
        doe = mommy.make(models.Contact, firstname="Jack", lastname="Doedoedoe")

        master = mommy.make(models.RelationshipType, name="Master", reverse="Padawan")

        models.Relationship.objects.create(contact1=obi, contact2=anakin, relationship_type=master)

        url = reverse('search')

        data = {"gr0-_-contacts_by_relationship_type-_-0": -master.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, obi.lastname)
        self.assertContains(response, anakin.lastname)
        self.assertNotContains(response, doe.lastname)

    def test_relationship_type(self):
        """contacts by relationship type : no reverse"""
        john = mommy.make(models.Contact, firstname="John", lastname="Lennon#")
        ringo = mommy.make(models.Contact, firstname="Georges", lastname="Harrison")
        doe = mommy.make(models.Contact, firstname="Jack", lastname="Doedoedoe")

        friends = mommy.make(models.RelationshipType, name="Fiends")

        models.Relationship.objects.create(contact1=john, contact2=ringo, relationship_type=friends)

        url = reverse('search')

        data = {"gr0-_-contacts_by_relationship_type-_-0": friends.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, john.lastname)
        self.assertContains(response, ringo.lastname)
        self.assertNotContains(response, doe.lastname)

    def test_relationship_date(self):
        """contacts by relationship date"""
        john = mommy.make(models.Contact, firstname="John", lastname="Lennon")
        ringo = mommy.make(models.Contact, firstname="Georges", lastname="Harrison")
        doe = mommy.make(models.Contact, firstname="Jack", lastname="Doedoedoe")

        friends = mommy.make(models.RelationshipType, name="Fiends")

        models.Relationship.objects.create(contact1=john, contact2=ringo, relationship_type=friends)

        url = reverse('search')

        today = date.today()
        data = {"gr0-_-contacts_by_relationship_dates-_-0": '{0} {0}'.format(today.strftime("%d/%m/%Y"))}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, john.lastname)
        self.assertContains(response, ringo.lastname)
        self.assertNotContains(response, doe.lastname)

    def test_relationship_not_in_date(self):
        """contacts by relationship not in date"""
        john = mommy.make(models.Contact, firstname="John", lastname="Lennon")
        ringo = mommy.make(models.Contact, firstname="Georges", lastname="Harrison")
        doe = mommy.make(models.Contact, firstname="Jack", lastname="Doedoedoe")

        friends = mommy.make(models.RelationshipType, name="Fiends")

        models.Relationship.objects.create(contact1=john, contact2=ringo, relationship_type=friends)

        url = reverse('search')

        tomorrow = date.today() + timedelta(1)
        data = {"gr0-_-contacts_by_relationship_dates-_-0": '{0} {0}'.format(tomorrow.strftime("%d/%m/%Y"))}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, john.lastname)
        self.assertNotContains(response, ringo.lastname)
        self.assertNotContains(response, doe.lastname)



# -*- coding: utf-8 -*-
"""Crm forms"""

from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext, ugettext_lazy as _

import floppyforms.__future__ as forms

from balafon.Crm import models
from balafon.Crm.forms.base import ModelFormWithAddress, FormWithFieldsetMixin
from balafon.Crm.settings import show_billing_address, NO_ENTITY_TYPE
from balafon.Crm.widgets import EntityAutoComplete


class EntityForm(FormWithFieldsetMixin, ModelFormWithAddress):
    """Edit entity form"""

    class Meta:
        """form is defined from model"""
        model = models.Entity
        fields = (
            'type', 'name', 'description', 'relationship_date', 'website', 'email', 'phone', 'fax',
            'street_number', 'street_type', 'address', 'address2', 'address3', 'zip_code', 'city', 'cedex', 'country',
            'logo', 'billing_street_number', 'billing_street_type', 'billing_address', 'billing_address2',
            'billing_address3', 'billing_zip_code', 'billing_city', 'billing_cedex', 'billing_country',
        )
        fieldsets = [
            ('name', {
                'fields': [
                    'type', 'name', 'description', 'relationship_date', 'website', 'email', 'phone', 'fax'
                ],
                'legend': _('Name')
            }),
            ('address', {
                'fields': [
                    'street_number', 'street_type', 'address', 'address2', 'address3', 'zip_code', 'city',
                    'cedex', 'country'
                ],
                'legend': _('Address')
            }),
            ('billing_address', {
                'fields': [
                    'billing_street_number', 'billing_street_type', 'billing_address', 'billing_address2',
                    'billing_address3', 'billing_zip_code', 'billing_city', 'billing_cedex', 'billing_country'
                ],
                'legend': _('Billing address')
            }),
            ('logo', {'fields': ['logo'], 'legend': _('Logo')}),
        ]

    def __init__(self, *args, **kwargs):
        super(EntityForm, self).__init__(*args, **kwargs)

        if NO_ENTITY_TYPE:
            self.fields["type"].widget = forms.HiddenInput()

    def get_fieldsets(self):
        """return the list of fieldsets"""

        for fieldset in self.Meta.fieldsets:
            if fieldset[0] == 'billing_address' and not show_billing_address():
                # Ignore 'Billing address' tab if disable in settings
                continue
            yield fieldset
    
    def clean_logo(self):
        """logo validation"""
        logo = self.cleaned_data["logo"]
        instance = self.instance
        if not instance:
            instance = ""
            try:
                instance.id = models.Entity.objects.latest('id').id
            except models.Entity.DoesNotExist:
                instance.id = 1
        target_name = models.get_entity_logo_dir(instance, logo)
        if len(target_name) >= models.Entity._meta.get_field('logo').max_length:
            raise ValidationError(ugettext("The file name is too long"))
        return logo


class EntityTypeForm(forms.ModelForm):
    """form for EntityType"""

    class Meta:
        """form from model"""
        model = models.EntityType
        fields = ('name', 'gender', 'order', 'logo', 'subscribe_form',)


class EntityRoleForm(forms.ModelForm):
    """form for entity role"""

    class Meta:
        """form from model"""
        model = models.EntityRole
        fields = ('name', )


class SelectEntityForm(forms.Form):
    """Select an entity"""
    entity = forms.CharField(label=_("Entity"))

    def __init__(self, *args, **kwargs):
        super(SelectEntityForm, self).__init__(*args, **kwargs)
        self.fields["entity"].widget = EntityAutoComplete(
            attrs={'placeholder': _('Enter the name of an entity'), 'size': '50', 'class': 'colorbox'})

    def clean_entity(self):
        """entity validation"""
        try:
            entity_id = int(self.cleaned_data["entity"])
            return models.Entity.objects.get(id=entity_id)
        except (ValueError, models.Entity.DoesNotExist):
            raise ValidationError(ugettext("The entity does'nt exist"))


class ChangeContactEntityForm(forms.Form):
    """Switch contact entity form"""
    OPTION_ADD_TO_EXISTING_ENTITY = 1
    OPTION_CREATE_NEW_ENTITY = 2
    OPTION_SWITCH_SINGLE_CONTACT = 3
    OPTION_SWITCH_ENTITY_CONTACT = 4

    OPTION_CHOICES = (
        (0, ""),
        (OPTION_ADD_TO_EXISTING_ENTITY, _("Reassign to an existing entity")),
        (OPTION_CREATE_NEW_ENTITY, _("Create a new entity")),
        (OPTION_SWITCH_SINGLE_CONTACT, _("Switch to single contact")),
        (OPTION_SWITCH_ENTITY_CONTACT, _("Switch to entity contact")),
    )

    option = forms.ChoiceField(label=_("What to do?"))
    entity = forms.IntegerField(
        label=_("Which one?"),
        required=False,
        widget=EntityAutoComplete(
            attrs={'placeholder': _('Enter the name of the entity'), 'size': '50', 'class': 'colorbox'}
        )
    )

    def __init__(self, contact, *args, **kwargs):
        self.contact = contact
        super(ChangeContactEntityForm, self).__init__(*args, **kwargs)

        if contact.entity.is_single_contact:
            single_contact_choices = (self.OPTION_CREATE_NEW_ENTITY, self.OPTION_SWITCH_SINGLE_CONTACT)
            choices = [choice for choice in self.OPTION_CHOICES if choice[0] not in single_contact_choices]
        else:
            choices = [choice for choice in self.OPTION_CHOICES if choice[0] != self.OPTION_SWITCH_ENTITY_CONTACT]

        self.fields['option'].choices = choices

        self.meth_map = {
            self.OPTION_ADD_TO_EXISTING_ENTITY: self._add_to_existing_entity,
            self.OPTION_CREATE_NEW_ENTITY: self._create_new_entity,
            self.OPTION_SWITCH_SINGLE_CONTACT: self._switch_single_contact,
            self.OPTION_SWITCH_ENTITY_CONTACT: self._switch_entity_contact,
        }

    def clean_option(self):
        """validation"""
        try:
            option = int(self.cleaned_data["option"])
            if option == 0:
                raise ValidationError(ugettext("Please select one of this options"))
            try:
                self.meth_map[option]
            except KeyError:
                raise ValidationError(ugettext("Invalid value"))
        except ValueError:
            raise ValidationError(ugettext("Invalid data"))
        return option

    def clean_entity(self):
        """validation"""
        option = self.cleaned_data.get("option", 0)
        if option != self.OPTION_ADD_TO_EXISTING_ENTITY:
            return None
        else:
            entity_id = self.cleaned_data["entity"]
            try:
                return models.Entity.objects.get(id=entity_id)
            except models.Entity.DoesNotExist:
                raise ValidationError(ugettext("Please select an existing entity"))

    def _add_to_existing_entity(self):
        """add to exsiting entity"""
        old_entity = self.contact.entity
        self.contact.entity = self.cleaned_data["entity"]
        self.contact.save()
        old_entity.save()

    def _create_new_entity(self):
        """create new entity"""
        old_entity = self.contact.entity
        self.contact.entity = models.Entity.objects.create()
        self.contact.save()
        old_entity.save()

    def _switch_single_contact(self):
        """switch to single contact"""
        old_entity = self.contact.entity
        self.contact.entity = models.Entity.objects.create(
            is_single_contact=True,
            name="{0.lastname} {0.firstname}".format(self.contact).lower()
        )
        self.contact.save()
        self.contact.entity.default_contact.delete()
        self.contact.entity.save()
        old_entity.save()

    def _switch_entity_contact(self):
        """switch to entity"""
        self.contact.entity.is_single_contact = False
        self.contact.entity.save()

    def change_entity(self):
        """change entity: call the method corresponding to the choice"""
        option = self.cleaned_data["option"]
        method = self.meth_map[option]
        method()



# -*- coding: utf-8 -*-
"""REST api powered by django-rest-framework"""

from __future__ import unicode_literals

from datetime import date, datetime, time

from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

from rest_framework import generics, viewsets
from rest_framework.exceptions import ParseError, PermissionDenied
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.views import APIView, Response

from balafon.Crm.models import Action, ActionType, ActionStatus, Contact, TeamMember, Entity
from balafon.Crm import serializers


class ContactViewSet(viewsets.ReadOnlyModelViewSet):
    """returns contacts for autocomplete"""
    permission_classes = [IsAdminUser, ]
    queryset = Contact.objects.all()
    serializer_class = serializers.ContactSerializer

    def get_queryset(self):
        """returns """
        name = self.request.GET.get('name', None)
        if name:
            return self.queryset.filter(lastname__istartswith=name)[:10]
        return self.queryset


class ContactsOrEntitiesView(APIView):
    """returns contacts or entity for autocomplete"""

    queryset = Contact.objects.all()
    serializer_class = serializers.ContactSerializer
    permission_classes = [IsAdminUser, ]

    def get_matching_items(self, name):
        """return list of contacts"""
        contacts_and_entities = []
        if name:
            contacts = Contact.objects.filter(lastname__istartswith=name)[:10]
            contact_type = ContentType.objects.get_for_model(Contact)
            entity_type = ContentType.objects.get_for_model(Entity)
            for contact in contacts:
                city = contact.get_city
                contacts_and_entities.append({
                    'id': contact.id,
                    'name': contact.fullname,
                    'type': contact_type.id,
                    'city': city.get_friendly_name() if city else '',
                })

            entities = Entity.objects.filter(name__istartswith=name, is_single_contact=False)[:10]
            for entity in entities:
                city = entity.city
                contacts_and_entities.append({
                    'id': entity.id,
                    'name': entity.name,
                    'type': entity_type.id,
                    'city': city.get_friendly_name() if city else '',
                })
        return sorted(contacts_and_entities, key=lambda elt: elt['name'])[:10]

    def get(self, request):
        """returns """
        name = self.request.GET.get('name', None)
        return Response(self.get_matching_items(name))


class ListActionsView(generics.ListAPIView):
    """update an action date"""
    permission_classes = [IsAdminUser, ]
    model = Action
    serializer_class = serializers.ActionSerializer

    def _parse_date(self, date_str):
        """string to date"""
        year, month, day = [int(val) for val in date_str.split("-")]
        return date(year, month, day)

    def _apply_in_charge_lookup(self, queryset):
        """apply a in_charge lookup if needed"""
        in_charge = self.request.GET.get('in_charge')
        if in_charge:
            try:
                in_charge_ids = [int(_id) for _id in in_charge.split(',')]
                team_members = TeamMember.objects.filter(id__in=in_charge_ids)
                if team_members.count() == 0:
                    return self.model.objects.none()
            except ValueError:
                raise ParseError("Invalid team member")
            queryset = queryset.filter(in_charge__in=[member.id for member in team_members])
        return queryset

    def _apply_in_action_type_lookup(self, queryset):
        """apply an action type lookup if needed"""
        action_type = self.request.GET.get('action_type')
        if action_type:
            try:
                ActionType.objects.get(id=action_type)
            except (ValueError, ActionType.DoesNotExist):
                raise ParseError("Invalid action type")
            queryset = queryset.filter(type__id=action_type)
        return queryset

    def get_queryset(self):
        """returns actions"""
        start_date = self.request.GET.get('start')
        end_date = self.request.GET.get('end')

        if not start_date or not end_date:
            raise ParseError("Period frame missing")

        queryset = self.model.objects.all()
        queryset = self._apply_in_action_type_lookup(queryset)
        queryset = self._apply_in_charge_lookup(queryset)

        try:
            start_date = self._parse_date(start_date)
            end_date = self._parse_date(end_date)
        except ValueError:
            raise ParseError("Invalid period frame")

        start_datetime = datetime.combine(start_date, time.min)
        end_datetime = datetime.combine(end_date, time.max)

        if end_datetime < start_datetime:
            return self.model.objects.none()

        queryset = queryset.filter(
            Q(planned_date__lte=start_datetime, end_datetime__gte=end_datetime) |  # starts before, ends after period
            Q(planned_date__gte=start_datetime, end_datetime__lte=end_datetime) |  # starts and ends during period
            Q(planned_date__lte=start_datetime, end_datetime__gte=start_datetime) |  # starts before, ends during
            Q(planned_date__lte=end_datetime, end_datetime__gte=end_datetime) |  # starts during period, ends after
            Q(
                planned_date__gte=start_datetime,
                end_datetime__isnull=True,
                planned_date__lte=end_datetime
            )  # no end, starts during period
        )
        return queryset


class ListTeamMemberActionsView(ListActionsView):
    """
    Get the actions of a current team member
    The team member doesn't have to be a balafon user
    """
    permission_classes = [IsAuthenticated]

    def _apply_in_charge_lookup(self, queryset):
        """apply a in_charge lookup if needed"""
        try:
            team_member = TeamMember.objects.get(user=self.request.user)
        except TeamMember.DoesNotExist:
            raise PermissionDenied
        return queryset.filter(in_charge=team_member)


class UpdateActionDateView(generics.UpdateAPIView):
    """update an action date"""
    model = Action
    serializer_class = serializers.ActionUpdateDateSerializer
    permission_classes = [IsAdminUser, ]

    def get_queryset(self):
        return self.model.objects.filter(pk=self.kwargs['pk'])


class EditActionMixin(object):
    """base class for create and update"""
    contacts = None

    def check_data(self, request):
        """check and prepare data"""
        self.contacts = request.data.get("contacts", [])
        action_type_id = request.data.get('type', None)
        action_status_id = request.data.get('status', None)

        if action_type_id and action_status_id:
            try:
                action_type = ActionType.objects.get(id=action_type_id)
                action_status = ActionStatus.objects.get(id=action_status_id)
                if action_status not in action_type.allowed_status.all():
                    raise ParseError(
                        "status {0} is not allowed for type {1}".format(action_type.name, action_status.name)
                    )
            except (ValueError, ActionType.DoesNotExist, ActionStatus.DoesNotExist):
                raise ParseError("Invalid parameters")

    def save_object(self, serializer):
        """save object"""
        contacts = []
        for contact_id in self.contacts:
            try:
                contact = Contact.objects.get(id=contact_id)
                contacts.append(contact)
            except (ValueError, Contact.DoesNotExist):
                raise ParseError("Invalid contacts")

        obj = serializer.save()
        obj.contacts.clear()
        for contact in contacts:
            obj.contacts.add(contact)
        obj.save()
        return obj


class UpdateActionView(EditActionMixin, generics.UpdateAPIView):
    """update an action"""
    permission_classes = [IsAdminUser, ]
    model = Action
    serializer_class = serializers.ActionUpdateSerializer
    contacts = None

    def get_queryset(self):
        return self.model.objects.filter(pk=self.kwargs['pk'])

    def update(self, request, *args, **kwargs):
        """handle HTTP request"""
        self.check_data(request)
        response = super(UpdateActionView, self).update(request, *args, **kwargs)
        return response

    def perform_update(self, serializer):
        """update the action"""
        return self.save_object(serializer)


class CreateActionView(EditActionMixin, generics.CreateAPIView):
    """create an action"""
    permission_classes = [IsAdminUser, ]
    model = Action
    serializer_class = serializers.ActionUpdateSerializer
    contacts = None

    def create(self, request, *args, **kwargs):
        """handle HTTP request"""
        self.check_data(request)
        response = super(CreateActionView, self).create(request, *args, **kwargs)
        return response

    def perform_create(self, serializer):
        """create new object"""
        return self.save_object(serializer)
        #obj.type = ActionType.objects.get_or_create(name="Intervention")[0]


class DeleteActionView(generics.DestroyAPIView):
    """delete an action"""
    model = Action
    permission_classes = [IsAdminUser, ]

    def get_queryset(self):
        return self.model.objects.filter(pk=self.kwargs['pk'])


class AboutMeView(generics.ListAPIView):
    """update an action date"""
    model = User
    serializer_class = serializers.UserSerializer
    permission_classes = [IsAuthenticated]

    def get_queryset(self):
        return self.model.objects.filter(id=self.request.user.id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0010_storeitem_published'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='brand',
            options={'ordering': ['name'], 'verbose_name': 'Brand', 'verbose_name_plural': 'Brands'},
        ),
        migrations.AddField(
            model_name='storeitem',
            name='origin',
            field=models.CharField(default=b'', max_length=50, verbose_name='Origine', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""
app configuration
"""

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from django.apps import AppConfig

class BalafonAppConfig(AppConfig):
    name = 'balafon.Emailing'
    verbose_name = _("Balafon Emailing")



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

from django.core.management.base import BaseCommand

from balafon.Crm import models


class Command(BaseCommand):
    help = "find the same as contacts"

    def add_arguments(self, parser):
        parser.add_argument('group_name', type=str, default="", nargs='?')

    def handle(self, group_name, *args, **options):
        verbose = options.get('verbosity', 0)

        group = None
        if group_name:
            group, is_new = models.Group.objects.get_or_create(name=group_name)
            if verbose > 0:
                print("create" if is_new else "update", "group", group_name)
        
        total_count = 0
        for index, contact in enumerate(models.Contact.objects.all()):
            if contact.lastname and contact.firstname:
                same_as = models.Contact.objects.filter(
                    lastname=contact.lastname, firstname=contact.firstname
                ).exclude(id=contact.id)
                same_as_count = same_as.count()
                if same_as_count > 0:
                    print(index, contact, same_as_count, "SameAs")
                    total_count += 1
                    if group:
                        group.contacts.add(contact)
                        group.save()
        
        if verbose > 0:
            print(total_count, "SameAs")



# -*- coding: utf-8 -*-

from balafon.permissions import can_access

from .models import UserPermissions


def can_create_group(user):
    """define who can access"""
    if can_access(user):
        try:
            preferences = UserPermissions.objects.get(user=user)
            return preferences.can_create_group
        except UserPermissions.DoesNotExist:
            return True  # allowed by default
    return False



# -*- coding: utf-8 -*-
"""bookmarks -> display on board: will be removed in future"""

from __future__ import unicode_literals

import json

from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404

from balafon.Crm import models
from balafon.permissions import can_access
from balafon.utils import logger


@user_passes_test(can_access)
def view_board_panel(request):
    """view"""
    return HttpResponseRedirect(reverse("users_favorites_list"))


def _toggle_object_bookmark(request, object_model, object_id):
    """util"""
    # pylint: disable=broad-except
    try:
        if request.is_ajax() and request.method == "POST":
            obj = get_object_or_404(object_model, id=object_id)
            obj.display_on_board = not obj.display_on_board
            obj.save()
            data = {'bookmarked': obj.display_on_board}
            return HttpResponse(json.dumps(data), 'application/json')
        raise Http404
    except Exception:
        logger.exception("_toggle_object_bookmarked")


@user_passes_test(can_access)
def toggle_action_bookmark(request, action_id):
    """view"""
    return _toggle_object_bookmark(request, models.Action, action_id)


@user_passes_test(can_access)
def toggle_opportunity_bookmark(request, opportunity_id):
    """view"""
    return _toggle_object_bookmark(request, models.Opportunity, opportunity_id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0002_auto_20160129_1527'),
    ]

    operations = [
        migrations.AddField(
            model_name='actiontype',
            name='hide_contacts_buttons',
            field=models.BooleanField(default=False, help_text='The add and remove contact buttons will be hidden', verbose_name='hide contacts buttons'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import floppyforms.__future__ as forms


class DatespanInput(forms.TextInput):
    """widget with two dates"""
    template_name = 'Search/_datespan_widget.html'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

from datetime import date

from django.core.management.base import BaseCommand
from balafon.Crm import models


class Command(BaseCommand):
    help = u"change the entity type"
    use_argparse = False

    def handle(self, *args, **options):
        verbose = options.get('verbosity', 0)

        if len(args) < 1:
            print("usage : change_entity_type new_type_name [old_type_id] [group=group_id]")
            return
        
        new_type_name = args[0]
        filter = args[1]
        if filter.find('group=')==0:
            gr_id = filter[6:]
            qs = models.Group.objects.get(id=gr_id).entities.all()
        else:
            qs = models.Entity.objects.filter(type__id=filter)
        
        entity_type, is_new = models.EntityType.objects.get_or_create(name=new_type_name)
        
        if is_new and verbose:
            print("Entity type", new_type_name, "has been created")
        
        for e in qs:
            e.type = entity_type
            e.save()
        
        if verbose:
            print(qs.count(), "entities updated")



# -*- coding: utf-8 -*-
"""common place for all Emailing settings"""

from __future__ import unicode_literals

from django.conf import settings as project_settings


def is_registration_enabled():
    """return true if subscribe page is enabled : True by default for compatibility reason"""
    return getattr(project_settings, 'BALAFON_REGISTRATION_ENABLED', True)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import url

from . import views


urlpatterns = [
    url(r'^newsletters/$', views.newsletter_list, name='emailing_newsletter_list'),
    url(r'^view-emailing/(?P<emailing_id>\d+)/$', views.view_emailing, name='emailing_view'),
    url(
        r'^update-emailing/(?P<pk>\d+)/$',
        views.EmailingUpdateView.as_view(),
        name='emailing_update_emailing'
    ),
    url(r'^delete-emailing/(?P<emailing_id>\d+)/$', views.delete_emailing, name='emailing_delete'),
    url(r'^new-newsletter/$', views.new_newsletter, name='emailing_new_newsletter'),
    url(r'^confirm-send-mail/(?P<emailing_id>\d+)/$', views.confirm_send_mail, name='emailing_confirm_send_mail'),
    url(r'^cancel_send_mail/(?P<emailing_id>\d+)/$', views.cancel_send_mail, name='emailing_cancel_send_mail'),
    url(r'^subscribe/$', views.SubscribeView.as_view(), name='emailing_subscribe_newsletter'),

    url(r'^email-subscribe/$', views.EmailSubscribeView.as_view(), name='emailing_email_subscribe_newsletter'),
    url(
        r'^email-subscribe/(?P<subscription_type>\d+)/$',
        views.EmailSubscribeView.as_view(),
        name='emailing_email_subscribe_subscription'
    ),
    url(r'^email-subscribe/done/$', views.EmailSubscribeDoneView.as_view(), name='emailing_subscribe_email_done'),
    url(r'^email-subscribe/error/$', views.EmailSubscribeErrorView.as_view(), name='emailing_subscribe_email_error'),
    url(
        r'^subscribe/(?P<contact_uuid>[\w\d-]+)$',
        views.subscribe_done,
        name='emailing_subscribe_done'
    ),
    url(
        r'^subscribe-error/(?P<contact_uuid>[\w\d-]+)/$',
        views.subscribe_error,
        name='emailing_subscribe_error'
    ),
    url(
        r'^email-verification/(?P<contact_uuid>[\w\d-]+)/$',
        views.email_verification,
        name='emailing_email_verification'
    ),
    url(
        r'^unregister/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.unregister_contact,
        name='emailing_unregister'
    ),
    url(
        r'^view-online/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.view_emailing_online,
        name='emailing_view_online'
    ),
    url(
        r'^view-online-lang/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/(?P<lang>\w+)/$',
        views.view_emailing_online_lang,
        name='emailing_view_online_lang'
    ),
    url(
        r'^link/(?P<link_uuid>[\w\d-]+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.view_link,
        name='emailing_view_link'
    ),
    url(
        r'^email-img/(?P<emailing_id>\d+)/(?P<contact_uuid>[\w\d-]+)/$',
        views.email_tracking,
        name='emailing_email_tracking'
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-24 10:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0025_actionstatus_allowed_on_frozen'),
    ]

    operations = [
        migrations.CreateModel(
            name='ActionNumberGenerator',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=200, verbose_name='name')),
                ('number', models.IntegerField(default=0, help_text='This number is auto-generated.', verbose_name='number')),
            ],
            options={
                'verbose_name': 'action number generator',
                'verbose_name_plural': 'action number generators',
            },
        ),
        migrations.AlterModelOptions(
            name='actionmenu',
            options={'ordering': ['order_index'], 'verbose_name': 'action menu', 'verbose_name_plural': 'action menus'},
        ),
        migrations.AddField(
            model_name='actiontype',
            name='number_generator',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='Crm.ActionNumberGenerator', verbose_name='number generator'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0008_auto_20161109_1714'),
    ]

    operations = [
        migrations.AddField(
            model_name='storemanagementactiontype',
            name='references_text',
            field=models.TextField(default=b'', help_text='this text will be added at the bottom of the commercial document', verbose_name='references text', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0006_storeitem_only_for_groups'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleitem',
            name='no_quantity',
            field=models.BooleanField(default=False, help_text='quantity and unit price are not shown on bill', verbose_name='no_quantity'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-09 13:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0024_actionstatus_next_on_send'),
    ]

    operations = [
        migrations.AddField(
            model_name='actionstatus',
            name='allowed_on_frozen',
            field=models.BooleanField(default=True, help_text='If False, not shown if action si frozen', verbose_name='allowed on frozen'),
        ),
    ]



# -*- coding: utf-8 -*-
"""Crm forms"""

from __future__ import unicode_literals

import floppyforms.__future__ as forms

from balafon.Crm import models
from balafon.Crm.forms.base import BetterBsForm


class CustomFieldForm(BetterBsForm):
    """base form for custom fields"""
    def __init__(self, instance, *args, **kwargs):
        super(CustomFieldForm, self).__init__(*args, **kwargs)
        self._instance = instance
        model_type = self._get_model_type()
        custom_fields = models.CustomField.objects.filter(model=model_type)
        for field in custom_fields:
            self.fields[field.name] = forms.CharField(required=False, label=field.label or field.name)
            if field.choices.count():
                choices = [('', '')] + [(choice.value, choice.label) for choice in field.choices.all()]
                self.fields[field.name].widget = forms.Select(choices=choices)
            self.fields[field.name].widget.attrs = {'class': 'form-control'}
            if field.widget:
                self.fields[field.name].widget.attrs['class'] += ' ' + field.widget

            # No Post
            if len(args) == 0:
                self.fields[field.name].initial = getattr(instance, 'custom_field_' + field.name, '')

    def save(self, *args, **kwargs):
        """save"""
        for field in self.fields:
            model_type = self._get_model_type()
            custom_field = models.CustomField.objects.get(name=field, model=model_type)
            custom_field_value = self._create_custom_field_value(custom_field)
            custom_field_value.value = self.cleaned_data[field]
            custom_field_value.save()
        return self._instance


class EntityCustomFieldForm(CustomFieldForm):
    """form for setting custom fields on an entity"""

    def _get_model_type(self):
        """is entity"""
        return models.CustomField.MODEL_ENTITY

    @staticmethod
    def model():
        """model"""
        return models.Entity

    def _create_custom_field_value(self, custom_field):
        """save the value in database"""
        custom_field_value = models.EntityCustomFieldValue.objects.get_or_create(
            entity=self._instance, custom_field=custom_field
        )[0]
        return custom_field_value


class ContactCustomFieldForm(CustomFieldForm):
    """form for setting custom fields of a contact"""

    def _get_model_type(self):
        """type -> contact"""
        return models.CustomField.MODEL_CONTACT

    def _create_custom_field_value(self, custom_field):
        """save value in database"""
        custom_field_value = models.ContactCustomFieldValue.objects.get_or_create(
            contact=self._instance, custom_field=custom_field
        )[0]
        return custom_field_value

    @staticmethod
    def model():
        """model"""
        return models.Contact


# -*- coding: utf-8 -*-
"""permission rules and utilities"""

from __future__ import unicode_literals


def can_access(user):
    """define who can access"""
    return user.is_authenticated() and user.is_staff


def is_admin(user):
    """define who can access"""
    return user.is_authenticated() and user.is_superuser



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0004_saleanalysiscode_action_type'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='sale',
            options={'ordering': ['-action__created'], 'verbose_name': 'Sale', 'verbose_name_plural': 'Sales'},
        ),
        migrations.AlterModelOptions(
            name='saleanalysiscode',
            options={'ordering': ('name',), 'verbose_name': 'Analysis code', 'verbose_name_plural': 'Analysis codes'},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Profile', '0002_auto_20160129_1527'),
    ]

    operations = [
        migrations.AddField(
            model_name='contactprofile',
            name='groups_ids',
            field=models.CharField(default=b'', max_length=100, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""a simple store"""

from __future__ import unicode_literals

from datetime import datetime, date, time
import xlrd
import xlwt

from django.core.urlresolvers import reverse, reverse_lazy
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from django.views.generic import FormView

from balafon.generic import XlsExportView, StaffPopupFormView
from balafon.Crm.models import ActionType
from balafon.Store.forms import StockImportForm
from balafon.Store.models import StoreItem
from balafon.Store.utils import to_decimal

from ..forms import DateRangeForm


class XlsBaseView(XlsExportView):

    def get_header_style(self):
        """style of header"""
        style = xlwt.XFStyle()
        style.pattern = xlwt.Pattern()
        style.pattern.pattern = xlwt.Pattern.SOLID_PATTERN
        style.pattern.pattern_fore_colour = 22
        return style


class StockXlsView(XlsBaseView):
    doc_name = 'export-stock.xls'

    def get_store_items(self):
        """returns objects to export"""
        return StoreItem.objects.all().order_by('category__name', 'name')

    def do_fill_workbook(self, workbook):
        """export to excel all articles tin stock"""

        sheet = workbook.add_sheet(_("Stock"))

        line = 0

        columns = [
            _('Id'), _('Name'), _('Category'), _('Purchase price'), _("Stock count"), _('Stock threshold'),
            _('Value'),
        ]

        for col, label in enumerate(columns):
            self.write_cell(sheet, 0, col, label, style=self.get_header_style())

        for line, item in enumerate(self.get_store_items()):
            sheet.write(line + 1, 0, item.id)
            sheet.write(line + 1, 1, item.name)
            sheet.write(line + 1, 2, '{0}'.format(item.category) if item.category else '')
            sheet.write(line + 1, 3, item.purchase_price if item.purchase_price else 0)
            sheet.write(line + 1, 4, '' if item.stock_count is None else item.stock_count)
            sheet.write(line + 1, 5, '' if item.stock_count is None else item.stock_threshold)
            sheet.write(line + 1, 6, xlwt.Formula('D{0}*E{0})'.format(line + 2)))

        sheet.write(line + 3, 6, xlwt.Formula('SUM(G1:G{0})'.format(line + 2)))


class StockAlertXlsView(StockXlsView):
    doc_name = 'export-stock-alert.xls'

    def get_store_items(self):
        """returns objects to export"""
        for item in StoreItem.objects.filter(stock_count__isnull=False):
            if item.has_stock_threshold_alert():
                yield item


class StoreXlsCatalogueView(XlsBaseView):
    doc_name = 'catalogue.xls'

    def do_fill_workbook(self, workbook):
        """implement it in base class"""

        sheet = workbook.add_sheet(_('Catalogue'))

        try:
            category_id = int(self.kwargs.get('category_id', 0))
        except ValueError:
            category_id = 0

        queryset = StoreItem.objects.filter(available=True)
        if category_id:
            queryset = queryset.filter(category=category_id)

        columns = [
            _('Name'), _('Category'), _('Brand'), _("VAT inclusive price"), _('Reference'),
            _('certificates'),
        ]

        for col, label in enumerate(columns):
            self.write_cell(sheet, 0, col, label, style=self.get_header_style())

        for line, store_item in enumerate(queryset):
            self.write_cell(sheet, line + 1, 0, store_item.fullname())
            self.write_cell(sheet, line + 1, 1, store_item.category.name if store_item.category else '')
            self.write_cell(sheet, line + 1, 2, store_item.brand.name if store_item.brand else '')
            self.write_cell(sheet, line + 1, 3, store_item.vat_incl_price())
            self.write_cell(sheet, line + 1, 4, store_item.reference)
            self.write_cell(
                sheet, line + 1, 5, ','.join(
                    [certificate.name for certificate in store_item.certificates.all()]
                )
            )


class StoreItemXlsView(XlsBaseView):
    doc_name = 'articles.xls'

    def do_fill_workbook(self, workbook):
        """implement it in base class"""

        sheet = workbook.add_sheet(_('Articles'))

        try:
            category_id = int(self.kwargs.get('category_id', 0))
        except ValueError:
            category_id = 0

        try:
            supplier_id = int(self.kwargs.get('supplier_id', 0))
        except ValueError:
            supplier_id = 0

        queryset = StoreItem.objects.all()
        if category_id:
            queryset = queryset.filter(category=category_id)

        if supplier_id:
            queryset = queryset.filter(supplier=supplier_id)

        columns = [
            _('Name'), _('Famille'), _('Category'), _('Brand'), _("TTC"), _("HT"), _('TVA'), _("Available"),
            _('Supplier'), _('Reference'), _('certificates'),
        ]

        for col, label in enumerate(columns):
            self.write_cell(sheet, 0, col, label, style=self.get_header_style())

        for line, store_item in enumerate(queryset):
            root_category = store_item.root_category()
            self.write_cell(sheet, line + 1, 0, store_item.fullname())
            self.write_cell(sheet, line + 1, 1, root_category.name if root_category else '')
            self.write_cell(sheet, line + 1, 2, store_item.category.name if store_item.category else '')
            self.write_cell(sheet, line + 1, 3, store_item.brand.name if store_item.brand else '')
            self.write_cell(sheet, line + 1, 4, store_item.vat_incl_price())
            self.write_cell(sheet, line + 1, 5, store_item.pre_tax_price)
            self.write_cell(sheet, line + 1, 6, str(store_item.vat_rate) if store_item.vat_rate else '')
            self.write_cell(sheet, line + 1, 7, _('Yes') if store_item.available else _('No'))
            self.write_cell(sheet, line + 1, 8, store_item.supplier.name if store_item.supplier else '')
            self.write_cell(sheet, line + 1, 9, store_item.reference)
            self.write_cell(
                sheet, line + 1, 10, ','.join(
                    [certificate.name for certificate in store_item.certificates.all()]
                )
            )


class StockImportView(FormView):
    """Import purchase price, stock and threshold of store items from Excel file"""
    form_class = StockImportForm
    template_name = "Store/import_stock.html"
    success_url = reverse_lazy('admin:Store_storeitem_changelist')

    def _get_value(self, value):
        if value is None or value == '':
            return None
        return to_decimal(value)

    def form_valid(self, form):
        """Process the file"""

        xls_file = form.cleaned_data['xls_file']

        workbook = xlrd.open_workbook(file_contents=xls_file.read())

        worksheet = workbook.sheet_by_index(0)

        for row in range(1, worksheet.nrows):
            item_id = worksheet.cell(rowx=row, colx=0).value
            if item_id:
                try:
                    store_item = StoreItem.objects.get(id=int(item_id))
                except (StoreItem.DoesNotExist, ValueError):
                    store_item = None

                if store_item:
                    store_item.purchase_price = self._get_value(worksheet.cell(rowx=row, colx=3).value)
                    store_item.stock_count = self._get_value(worksheet.cell(rowx=row, colx=4).value)
                    store_item.stock_threshold = self._get_value(worksheet.cell(rowx=row, colx=5).value)
                    store_item.save()

        return super(StockImportView, self).form_valid(form)


class ActionSummaryFormView(StaffPopupFormView):
    template_name = 'Store/popup_actions_summary.html'
    form_class = DateRangeForm
    action_type = None

    def get_header_style(self):
        """style of header"""
        style = xlwt.XFStyle()
        style.pattern = xlwt.Pattern()
        style.pattern.pattern = xlwt.Pattern.SOLID_PATTERN
        style.pattern.pattern_fore_colour = 22
        return style

    def get_action_type(self):
        if self.action_type is None:
            self.action_type = get_object_or_404(ActionType, id=self.kwargs['action_type_id'])
        return self.action_type

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['action_type'] = self.get_action_type()
        return context

    def form_valid(self, form):
        """returns objects to export"""
        action_type = self.get_action_type()
        start_date = form.cleaned_data['start_date'].strftime('%Y-%m-%d')
        end_date = form.cleaned_data['end_date'].strftime('%Y-%m-%d')
        url = reverse('store_actions_summary_xls', args=[action_type.id, start_date, end_date])
        return HttpResponseRedirect(url)


class ActionSummaryXlsView(XlsBaseView):
    doc_name = 'action-summary.xlsx'
    template_name = 'Store/popup_actions_summary.html'
    form_class = DateRangeForm
    only_staff = True
    action_type = None

    def get_action_type(self):
        if self.action_type is None:
            self.action_type = get_object_or_404(ActionType, id=self.kwargs['action_type_id'])
        return self.action_type

    def get_actions(self, action_type, start_date, end_date):
        """returns objects to export"""
        start_date = date(*[int(elt) for elt in start_date.split('-')])
        end_date = date(*[int(elt) for elt in end_date.split('-')])

        start_datetime = datetime.combine(start_date, time.min)
        end_datetime = datetime.combine(end_date, time.max)

        return action_type.action_set.filter(
            planned_date__gte=start_datetime, planned_date__lte=end_datetime
        ).order_by('number')

    def do_fill_workbook(self, workbook):
        """export to excel all actions of this type"""

        action_type = self.get_action_type()
        start_date = self.kwargs['start_date']
        end_date = self.kwargs['end_date']

        name = action_type.name
        sheet = workbook.add_sheet(name)
        self.doc_name = "{0}_{1}_{2}.xlsx".format(name.lower().replace(' ', '-'), start_date, end_date)

        columns = [
            _('Number'), _('Client'), _('Date') + ' ' * 6, _('State'), _('Accounting code'), _('Pre-tax amount'), _('VAT amount'),
            _('Tax incl Amount'),
        ]

        sheet.col(2).width = 256 * 20

        date_format = xlwt.XFStyle()
        date_format.num_format_str = 'dd/mm/yyyy'

        for col, label in enumerate(columns):
            self.write_cell(sheet, 0, col, label, style=self.get_header_style())

        actions = self.get_actions(action_type, start_date, end_date)
        line = 0
        for action in actions:

            for accounting_elt in action.sale.accounting_code_amounts():
                line += 1

                sheet.write(line, 0, action.number)
                sheet.write(line, 1, action.get_recipients(False))
                sheet.write(line, 2, action.planned_date, style=date_format)
                sheet.write(line, 3, action.status.name if action.status else '')
                sheet.write(line, 4, accounting_elt['accounting_code'])
                sheet.write(line, 5, accounting_elt['pre_tax_total'])
                sheet.write(line, 6, accounting_elt['vat_total'])
                sheet.write(line, 7, accounting_elt['vat_incl_total'])

        if line:
            sheet.write(line + 2, 5, xlwt.Formula('SUM(F2:F{0})'.format(line + 1)), style=self.get_header_style())
            sheet.write(line + 2, 6, xlwt.Formula('SUM(G2:G{0})'.format(line + 1)), style=self.get_header_style())
            sheet.write(line + 2, 7, xlwt.Formula('SUM(H3:H{0})'.format(line + 1)), style=self.get_header_style())



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-03-28 16:01
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('Users', '0003_auto_20180409_1301'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserPermissions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('can_create_group', models.BooleanField(default=True, help_text='If not checked, the user can not create group', verbose_name='can create group')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User permissions',
                'verbose_name_plural': 'User permissions',
            },
        ),
    ]
